function [time, perf] = SFW(X, Y, tau, testX, testY, budget)
% Stochastic Frank-Wolfe for solving multinomial logistic regression
% under a trace norm constraint
    
%   Input: 
%   X = feature vectors    
%   Y = labels
%   tau = trace norm bound

%   Output:     
    
    N = 50000;
    %m = 100;
    [n, d] = size(X);
    h = max(Y);
    W = zeros(h,d); %LMO(get_grad(zeros(h,d),X,Y,1:n), tau);
    
    step = 5;
    time = [];
    perf = [];
    time_elapsed = 0;

%     u = randn(h, 1);
%     u = u / norm(u);
%     v = randn(d, 1);
%     v = v / norm(v);	
    
    tic;    
    for k = 1:N            
        m = min(n, k^2);
                
        G = get_grad(W,X,Y,randsample(n,m)');                          
        V = LMO(G,tau);        
        %[u, v] = FastSingularVectors(G,u,v);
        %V = -tau * u * v';                
        
        gamma = 2 / (k+1);
        W = (1 - gamma) * W + gamma * V;
        
        if mod(k, step) == 0
            time_elapsed = time_elapsed + toc;
            time = [time; time_elapsed];
            perf = [perf; measure(W, X, Y, testX, testY)];
            fprintf('SFW: k = %d done\n', k); 
            
            if time_elapsed > budget                 
                return
            end
            tic;
        end
    end     
end
