function [time, perf] = SVRG(X, Y, tau, testX, testY, budget)
% Stochastic Variance Reduced Gradient Descent for solving multinomial 
% logistic regression under a trace norm constraint
    
%   Input: 
%   X = feature vectors    
%   Y = labels
%   tau = trace norm bound

%   Output:     
    
    T = 500;
    %N = 10;
    m = 100;
    [n, d] = size(X);
    h = max(Y);
    U = zeros(h,d); %LMO(get_grad(zeros(h,d),X,Y,1:n), tau);
    W = U;   
    
    time = [];
    perf = [];    
    time_elapsed = 0;
         
    tic;
    for t = 1:T                        
        N = 50; %2^(1+t);
        if t > 1
            FG = get_grad(W,X,Y,1:n);   
        end
        num = min(N, 20);
        step = floor(N / num);
        
        for k = 1:N            
            if k == 1 && t > 1
                G = FG;                
            else                
                sample = randsample(n,m)';
                GU = get_grad(U,X,Y,sample);
                if t < 2
                    G = GU;
                else
                    GW = get_grad(W,X,Y,sample);
                    G = GU - GW + FG;
                end
            end
            
            eta = .1; % 1 / sqrt(k);
            U = TraceProject(U - eta * G, tau);    
            
            if mod(k, step) == 0
                time_elapsed = time_elapsed + toc;
                time = [time; time_elapsed];
                perf = [perf; measure(U, X, Y, testX, testY)];
                
                if time_elapsed > budget                    
                    return
                end                
                tic;
            end
        end
        W = U;       

        fprintf('SVRG: t = %d done\n', t);
    end
end
